% Demo - background in fuzzy sets.

% (c) Robert Babuska, March 1994

echo off
% Reset things before we get started.
clear
clc; clf;
format
echo on
%       FUZZY SETS
%
% This demo introduces the concepts of fuzzy set, membership function,
% fuzzy set domain, kernel, support, etc. Representation of fuzzy
% sets in MATLAB is explained.

% Press any key to continue...
pause
clc
% In classical set theory a set (i.e. collection of objects with certain
% property) can be represent by its characteristic function:
%
%                 1     if element x is member of set A
%       Ax  = <
%                 0     if element x is not member of set A

% Press any key to continue...
pause
clc
% Fuzzy set is an extension of a classical set, in the sense that
% its characteristic function can take real values, usually from
% interval <0,1>. Elements with membership degree equal to:
%
%              1     - full members of set A
%       Ax =  (1,0)  - partial members of set A
%              0     - no membership

% Press any key to continue...
pause
clc
% In this tutorial we will use the following terminology and notation:
% The characteristic function of a fuzzy set is called MEMBERSHIP FUNCTION,
% its values are MEMBERSHIP GRADES or MEMBERSHIP DEGREES. In printed
% texts membership degrees (functions) are usually denoted by Greek
% letter 'mu' with the set name in subscript and element in argument, e.g.

%       mu (x)   or as in LATEX  \mu_A(x)
%         A

% Press any key to continue...
pause
clc
% Here we adopt a simple "computer" notation. Consider a domain
% (universe of discourse) X with elements x. If fuzzy set A is
% defined on this domain, 'Ax' denotes membership degree of element
% 'x' into the fuzzy set 'A'.

% For the sake of brevity the following abbreviations will be used:

%       MF, MFs - Membership function(s)
%       FS      - Fuzzy set

% Press any key to continue...
pause
clc
% Fuzzy Sets Representation in MATLAB
%
% A fuzzy set can be defined on a finite or infinite domain. On a finite
% domain a fuzzy set can be represented as set of pairs

% {membership grade/element}, i.e. {Ax/x; for x from X)

% In MATLAB, we represent this set by two vectors X and Ax. X
% contains the elements and Ax their respective membership degrees.

% Press any key to continue...
pause
clc
% EXAMPLE:
% X is a set of positive integers, A is a fuzzy set of integers
% approximately equal to 8. We can write A as

% A = 0.1/5 + 0.5/6 + 0.8/7 + 1/8 + 0.8/9 + 0.5/10 + 0.1/11

% In MATLAB commands:

Ax = [0 .1 .5 .8  1 .8  .5  .1   0];
X  = [4, 5, 6, 7, 8, 9, 10, 11, 12];

% Press any key to continue...
pause
clc
% Let us plot this membership function:

plot(X,Ax); axis([0 15 0 1]);
title('Integers approximately equal to 8');
ylabel('Membership grade');

% Press any key to continue...
pause
clc
% Fuzzy sets on continuous domains are described by analytical membership
% functions. As an example consider a fuzzy set of real numbers
% approximately equal to 8. This fuzzy set can be defined by a
% bell-shaped membership function

% exp(-(x-c)^2/(2*a))

% where c and a are membership function center and spread respectively.

% Press any key to continue...
pause
clc
% In MATLAB commands:

X = 5:0.05:10;                  % discretized real interval
c = 8;                          % membership function center
a =.1;                          % membership function spread
Ax = exp(-(X-c).^2/(2*a));

plot(X,Ax); axis([5 10 0 1]);
title('Reals approximately equal to 8');
ylabel('Membership grade');

% Press any key to continue...
pause
clc
% In practical applications triangular and trapezoidal membership functions
% are often used, because of less computational demands. To facilitate
% definitions of membership functions of different types, function called
% MGRADE is available in the toolbox. The usage of this function is
% described in the part 'MEMBERSHIP FUNCTIONS'. For this and also other
% predefined functions an on-line help can be obtained by typing e.g.:

% help mgrade

% Press any key to continue...
pause
clc
% Now let us review some elementary definitions from fuzzy set theory.
% Consider fuzzy set A defined on universe X:
A = [0 .2 .5 .8  1  1];
X = [2  4  6  8 10 12];
% SUPPORT of a fuzzy set is a classical set Supp A = {x; Ax > 0}
supp(X,A)

% Press any key to continue...
pause
clc
A = [0 .2 .5 .8  1  1];
X = [2  4  6  8 10 12];
% KERNEL of a fuzzy set is a classical set Ker A = {x; Ax = 1}
ker(X,A)

% Press any key to continue...
pause
clc
A = [0 .2 .5 .8  1  1];
X = [2  4  6  8 10 12];
% HEIGHT of a fuzzy set is a real number Hgt A = max(Ax)
hgt(A)

% Press any key to continue...
pause
clc
A = [0 .2 .5 .8  1  1];
X = [2  4  6  8 10 12];
% ALPHA-CUT of a fuzzy set is a classical set Aa = {x; Ax >= alpha}
alpha = 0.4;
acut(X,A,alpha)

% Press any key to quit...
pause
echo off; clc

